/************************************************************************************
*
* Copyright (c) 2003 The University of Texas
* All Rights Reserved.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.

* The GNU Public License is available in the file LICENSE, or you
* can write to the Free Software Foundation, Inc., 59 Temple Place -
* Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
* World Wide Web at http://www.fsf.org.
*
* IN NO EVENT SHALL THE UNIVERSITY OF TEXAS BE LIABLE TO ANY PARTY 
* FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES 
* ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF 
* THE UNIVERSITY OF TEXAS HAS BEEN ADVISED OF THE POSSIBILITY OF 
* SUCH DAMAGE.
*
* THE UNIVERSITY OF TEXAS SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
* PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
* TEXAS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
* ENHANCEMENTS, OR MODIFICATIONS.
*
* Programmers:  Mayank Gupta <mayank@mail.utexas.edu ; mayank42@hotmail.com>
* Advisor:      Prof. Brian L. Evans
* Version:      0.1
*
************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include "ROT_header.h"

short image[Blkrow][Blkcol]; /* The grey image block , global variable */
short **RAM_Rimage;			 /* The input red component of image , global variable */
short **RAM_Gimage;			 /* The input green component of image , global variable */
short **RAM_Bimage;			 /* The input blue component of image , global variable */

int main (void){
int r;				   /* number of rows  */
int c;				   /* number of cols  */
int maxval;			   /* maximum pixel value of input image */
int r_cg;			   /* row center of gravity  */
int c_cg;       	   /* col center of gravity  */
int numpixels=0;	   /* number of edge pixels in a block */
int nr,nc;      	   /* number of blocks across horizontal and vertical axis */
int sum_edge_pixels=0; /* total number of edge pixels in image */
int rcg_index = 0;	   /* nr counter in loop */
int ccg_index = 0;	   /* nc counter in loop */
int b = 0;			   /* count of number of blocks */	
clock_t start,stop,overhead,total; /* clock variables*/

int blkr = Blkrow;
int blkc = Blkcol;
int xtra_r;
int xtra_c;
char nof = 0;
int i,j;
int r_cg_sum = 0;
int c_cg_sum = 0;

total = 0;
start = clock();
stop = clock();
overhead = start - stop;

/************************************************************************************
 * Read in a raw ppm color image and load it into memory 
 ************************************************************************************/

ReadPpmImg(&r,&c,&maxval,&nof);
if (nof == 1) return 0;
start = clock();

/*****************************************************************************************
 * Determine number of blocks (nr*nc) to process
 *****************************************************************************************/

nr = r / blkr;
nc = c / blkc;

if ((nr * blkr) != r) {nr++; xtra_r = 1;} 
if ((nc * blkc) != c) {nc++; xtra_c = 1;}

printf("number of blocks along the row = %d\n",nr);
printf("number of blocks along the col = %d\n",nc);
printf("total number of blocks to process = %d\n",nc*nr);

/************************************************************************************
 * Begin processing by blocks
 ************************************************************************************/

for (rcg_index = 0 ; rcg_index < nr ; rcg_index++){
	for (ccg_index = 0 ; ccg_index < nc ; ccg_index++){

printf("\nStarting block %d\n",b+1);

/************************************************************************************
 * Read image block from memory into image[][]
 * Each image block is size Blkrow x Blkcol 
 ************************************************************************************/
if (rcg_index == nr-1){
	if (xtra_r == 1) blkr = (r - (nr-1)*Blkrow);}
if (ccg_index == nc-1){
	if (xtra_c == 1) blkc = (c - (nc-1)*Blkcol);}

//stop = clock();
//total += (stop-start-overhead);

//#pragma MUST_ITERATE(1,90);
for (i = 0; i <blkr; i++){
//#pragma MUST_ITERATE(1,90);
	for (j = 0; j<blkc; j++){
		image[i][j] = ((RAM_Rimage[(rcg_index*Blkrow)+i][(ccg_index*Blkcol)+j]) + 
					   (RAM_Gimage[(rcg_index*Blkrow)+i][(ccg_index*Blkcol)+j]) + 
					   (RAM_Bimage[(rcg_index*Blkrow)+i][(ccg_index*Blkcol)+j])) / 3;
}}
//start = clock();

//***************************************************************************************************
//* Detect edge and number of edge pixels of filtered grey-scale image block 
//****************************************************************************************************/

FilterDetectEdge(blkr,blkc,&numpixels);
//printf("Edge detection complete \n");
	
//************************************************************************************
// * Find center of gravity of grey-scale image block 
//************************************************************************************/
if (numpixels > 25){
	CenterOfGravity(blkr,blkc,&r_cg,&c_cg);
	
	//printf("row cg for block = %d\n",r_cg);
	//printf("col cg for block = %d\n",c_cg);
   	
   	printf("over all row cg for block = %d\n",(r_cg + (rcg_index*Blkrow)));
	printf("over all col cg for block = %d\n",(c_cg + (ccg_index*Blkcol)));
	printf("number of edge pixels in the block (weight) = %d\n",numpixels);
   	
   	r_cg_sum += (r_cg + (rcg_index*Blkrow)) * numpixels; 
   	c_cg_sum += (c_cg + (ccg_index*Blkcol)) * numpixels; 
   	
   	sum_edge_pixels += numpixels;
   	printf("Finished block %d\n",b+1);
				 	}
else printf("Block does not contain main subject\n");

b++;
numpixels = 0;
c_cg = 0;
r_cg = 0;	
blkr = Blkrow;
blkc = Blkcol;
}}

//***********************************************************************************
//* End processing by blocks
//* Calculate center of gravity of entire image
//************************************************************************************/

if (sum_edge_pixels != 0){
	r_cg = r_cg_sum / sum_edge_pixels;	
	c_cg = c_cg_sum / sum_edge_pixels;	
	printf("\nrow cg for the image= %d\n",r_cg);
	printf("col cg for the image= %d\n",c_cg);

//************************************************************************************
//* Circular shift and write final color image that obeys rule of third in ppm format
//************************************************************************************/

	WriteROTPpmImg(r,c,maxval,r_cg,c_cg);
				}
/* calculate total cycles for program execution */
stop = clock();
//printf("total cycles (not reading image) = %d\n",total);
printf("total cycles (not reading image) = %d\n",stop - start - overhead);
free(RAM_Rimage);
free(RAM_Gimage);
free(RAM_Bimage);
return 0;
}




