/************************************************************************************
*
* Copyright (c) 2003 The University of Texas
* All Rights Reserved.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.

* The GNU Public License is available in the file LICENSE, or you
* can write to the Free Software Foundation, Inc., 59 Temple Place -
* Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
* World Wide Web at http://www.fsf.org.
*
* IN NO EVENT SHALL THE UNIVERSITY OF TEXAS BE LIABLE TO ANY PARTY 
* FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES 
* ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF 
* THE UNIVERSITY OF TEXAS HAS BEEN ADVISED OF THE POSSIBILITY OF 
* SUCH DAMAGE.
*
* THE UNIVERSITY OF TEXAS SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
* PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
* TEXAS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
* ENHANCEMENTS, OR MODIFICATIONS.
*
* Programmers:  Mayank Gupta <mayank@mail.utexas.edu ; mayank42@hotmail.com>
* Advisor:      Prof. Brian L. Evans
* Version:      0.1
*
************************************************************************************/



/************************************************************************************
* The program reads ppm images that can be generated using XV, PixWizard or MATLAB.
*
* The images are read in the function ReadPpmImg.c.
* To specify the image to be read please change the name of the image in double quotations (" ") in the function ReadPpmImg.c:
*
* #define IMG "manblur_bxv.ppm"	/* name of input image 
*
* ppm images generated by different programs have different attributes.
* ReadPpmImg must be changed depending on which program was used to generate ppm image
*
* 1. For MATLAB and XV generated ppm images:Both of these lines must be present
*
* 	 fgetc(fp);  // This line must be added if using XV generated ppm images
* 	 if (fgetc(fp) =='#') fgets(cbuf,70,fp);	//This line is needed for MATLAB or XV generated ppm images not for pixwizard generated images
*
* 2. For PixWizard generated ppm images: The lines above must be commented out
***********************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include "ROT_header.h"
#define IMG "manblur_bxv.ppm"	/* name of input image */

extern short **RAM_Rimage;
extern short **RAM_Gimage;
extern short **RAM_Bimage;

/************************************************************************************
 * void WritePpmImgROT(int row, int col,int maxval, int rcg, int ccg)
 * This function reads a color ppm image (generated by MATLAB, XV or pixwizard) 
 * and stores each color component in a separate matrix. 
 * r = number of rows
 * c = number of columns
 ************************************************************************************/

void ReadPpmImg(int *r, int *c,int *maxval, char*nof){

FILE *fp;
char buf[3];
char cbuf[100];
int i,j;
int rd=0;
int gn=0;
int bl=0;
int gry=0;
int tempr,tempc,tempmaxval;

if ((fp = fopen(IMG, "rb")) == NULL){
	fprintf(stderr, "The file '%s' does not exist\n", IMG);
	*nof = 1;
	return ;}

fgets(buf, 3, fp);

if (strncmp(buf,"P6",2) != 0){ //P6 for binary format ppm P3 for ascii format ppm
	fprintf(stderr, "The file '%s' is not in correct PPM format\n",IMG);
	*nof = 1;
	return ;}

fgetc(fp);  // This line must be added if using XV generated ppm images

if (fgetc(fp) =='#') fgets(cbuf,70,fp);	//This line is needed for MATLAB or XV generated ppm images not for pixwizard generated images
	
fscanf(fp, "%d %d", &tempc, &tempr);
printf("row = %d\n",tempr);
printf("col = %d\n",tempc);

/* Allocate and read each color component of the image into 2-d arrays */
RAM_Rimage = (short**) malloc((tempr)*sizeof(short*)); 
RAM_Rimage[0] = (short*) malloc(((tempr)*(tempc))*sizeof(short));
for (i = 1; i < (tempr); i++) 
	RAM_Rimage[i] = RAM_Rimage[i-1] + (tempc);

RAM_Gimage = (short**) malloc((tempr)*sizeof(short*)); 
RAM_Gimage[0] = (short*) malloc(((tempr)*(tempc))*sizeof(short));
for (i = 1; i < (tempr); i++) 
	RAM_Gimage[i] = RAM_Gimage[i-1] + (tempc);

RAM_Bimage = (short**) malloc((tempr)*sizeof(short*)); 
RAM_Bimage[0] = (short*) malloc(((tempr)*(tempc))*sizeof(short));
for (i = 1; i < (tempr); i++) 
	RAM_Bimage[i] = RAM_Bimage[i-1] + (tempc);

fscanf(fp,"%d",&tempmaxval);
fgetc(fp);
for (i = 0; i <(tempr); i++){
	for (j = 0; j<(tempc); j++){
		RAM_Rimage[i][j] = fgetc(fp);
		RAM_Gimage[i][j] = fgetc(fp);
		RAM_Bimage[i][j] = fgetc(fp);
		}}

*r = tempr;
*c = tempc;
*maxval = tempmaxval;
fclose(fp);
printf("Image read and loaded into memory successfully \n");
return;
}
