
/************************************************************************************
*
* Copyright (c) 2003 The University of Texas
* All Rights Reserved.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.

* The GNU Public License is available in the file LICENSE, or you
* can write to the Free Software Foundation, Inc., 59 Temple Place -
* Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
* World Wide Web at http://www.fsf.org.
*
* IN NO EVENT SHALL THE UNIVERSITY OF TEXAS BE LIABLE TO ANY PARTY 
* FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES 
* ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF 
* THE UNIVERSITY OF TEXAS HAS BEEN ADVISED OF THE POSSIBILITY OF 
* SUCH DAMAGE.
*
* THE UNIVERSITY OF TEXAS SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
* PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
* TEXAS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
* ENHANCEMENTS, OR MODIFICATIONS.
*
* Programmers:  Mayank Gupta <mayank@mail.utexas.edu ; mayank42@hotmail.com>
* Advisor:      Prof. Brian L. Evans
* Version:      0.1
*
************************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include "ROT_header.h"

extern short **RAM_Rimage;
extern short **RAM_Gimage;
extern short **RAM_Bimage;

/************************************************************************************
 * void WritePpmImgROT(int row, int col,int maxval, int rcg, int ccg)
 * This function determines the degree of shift in the vertical and horizontal directions
 * and writes the circularly shifted color image that obeys the rule of third : outROT.pgm
 ************************************************************************************/
 
void WriteROTPpmImg(int r, int c,int maxval,int rcg, int ccg){

FILE *fp;
int i=0;
int j=0;
int v[8];
int mask_indx_c;
int mask_indx_r;
int vnew[4];	
char sm_indx;
int sm_val;
int a,b;

mask_indx_c = (c / 3);
mask_indx_r = (r / 3);

/* Determine direction and magnitude of shift */
v[0] = ((2*mask_indx_r) - rcg);
v[1] = (mask_indx_c - ccg);

v[2] = ((2*mask_indx_r) - rcg);
v[3] = ((2*mask_indx_c) - ccg);

v[4] = (mask_indx_r - rcg);
v[5] = ((2*mask_indx_c) - ccg);

v[6] = (mask_indx_r - rcg);
v[7] = (mask_indx_c - ccg);

vnew[0] = pow(v[0],2) + pow(v[1],2);
vnew[1] = pow(v[2],2) + pow(v[3],2);
vnew[2] = pow(v[4],2) + pow(v[5],2);
vnew[3] = pow(v[6],2) + pow(v[7],2);

sm_val = vnew[0];
sm_indx = 0;

if (sm_val > vnew[1]) {sm_indx = 2; sm_val = vnew[1];}
if (sm_val > vnew[2]) {sm_indx = 4; sm_val = vnew[2];}
if (sm_val > vnew[3]) sm_indx = 6;

a = v[sm_indx];
b = v[sm_indx+1]; 

printf("shift in row location of cg (a) = %d (positive means down)\n",a);
printf("shift in col location of cg (b) = %d (positive means right)\n",b);

fp = fopen("outROT.ppm", "wb"); 	
fprintf(fp, "P6\n%d %d",c,r);
fprintf(fp, " %d\n", maxval);

/* Shift and write rule of thirds image */
if(a < 0){
	if(b >= 0){
	
		for (i = (-a); i < r ; i++){			// -a +b
			for (j = (b-1); j >= 0 ; j--)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
			for (j = 0;j < (c-b) ;j++)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
						}

		for (i = r-1; i > r-1+a ; i--){
			for (j = (b-1); j >= 0; j--)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
			for (j = 0; j < c-b; j++)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
					}
				}
	else{
		 for (i = (-a); i < r ; i++){				// -a -b 
			for (j = -b; j < c; j++)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}

			for (j = c-1; j > c-1+b; j--)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
					}

		for (i = r-1; i > r-1+a ; i--){
			for (j = -b; j < c; j++)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
			for (j = c-1; j > c-1+b; j--)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
					}
		}
		}
if (a >= 0){
	if(b >= 0){
		for (i = a-1; i >= 0; i--){				// +a +b
			for (j = b-1; j >= 0; j--)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
			for (j = 0; j < c-b; j++)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
						}
									

		for (i = 0; i < (r-a); i++){
			for (j = (b-1); j >= 0; j--)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
			for (j = 0; j < c-b; j++)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
									}
			} 
	else{
		for (i = a-1; i >= 0 ; i--){				// +a -b 
			for (j = -b; j < c; j++)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
			for (j = c-1; j > c-1+b; j--)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
					}

		for (i = 0; i < r-a; i++){
			for (j = -b; j < c; j++)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
			for (j = c-1; j > c-1+b ; j--)
				{fputc(RAM_Rimage[i][j],fp);
				fputc(RAM_Gimage[i][j],fp);
				fputc(RAM_Bimage[i][j],fp);}
					}
		} 
	}

fclose(fp);
printf("Completed Rule of Thirds, image written\n");
return;
}
